import { isBoolean, isFunction, isArray } from '@/utils/is';
import { get } from 'lodash-es';
import type { Ref, ComputedRef } from 'vue';
import { ref, unref, watch, onMounted, computed } from 'vue';
import { FETCH_FIELD_SETTING } from '../constant';
import { PaginationProps } from '../types/pagination';
import { BasicTableProps, FetchParams } from '../types/table';

interface ActionType {
	setLoading: (loading: boolean) => void;
	getPaginationRef: ComputedRef<PaginationProps | boolean>;
}

export function useDataSource(
	propsRef: Ref<BasicTableProps>,
	{ setLoading, getPaginationRef }: ActionType,
	emit: EmitType,
) {
	const dataSourceRef = ref<Recordable[]>([]);
	const rawDataSourceRef = ref<Recordable>({});

	watch(
		() => unref(propsRef).dataSource,
		() => {
			const { api, dataSource } = unref(propsRef);
			if (!api && dataSource) {
				dataSourceRef.value = dataSource;
			}
		},
		{
			immediate: true,
		},
	);

	const getDataSourceRef = computed(() => {
		const dataSource = unref(dataSourceRef);
		if (!dataSource || dataSource.length === 0) {
			return [];
		}
		return unref(dataSourceRef);
	});

	const getRowKeyRef = computed(() => {
		const { rowKey } = unref(propsRef);
		// TODO: 支持自动创建key
		return rowKey;
	});

	// TODO: 表格分页，排序，过滤器发生变化
	// function handleTableChange(pagination, filters, sorter) {}

	/**
	 * 请求接口
	 */
	async function fetch(opts?: FetchParams) {
		const { api, loading, pagination, fetchFieldsSetting } = unref(propsRef);

		if (!api && !isFunction(api)) return;
		try {
			loading && setLoading(loading);

			const { pageField, pageSizeField, listField, totalField } = Object.assign(
				{},
				FETCH_FIELD_SETTING,
				fetchFieldsSetting,
			);

			// TODO: 分页信息的默认值，usePagination
			let pageParams: Recordable = {};
			const getPagination = unref(getPaginationRef);

			if ((isBoolean(pagination) && !pagination) || isBoolean(getPagination)) {
				pageParams = {};
			} else {
				const { current = 1, pageSize = 10 } = getPagination;
				// TODO: 字段的自定义与默认值，
				pageParams[pageField] = opts?.page || current;
				pageParams[pageSizeField] = pageSize;
			}

			let params = {
				...pageParams,
				...(opts?.sortInfo ?? {}),
				...(opts?.filterInfo ?? {}),
			};

			const rs = await api(params);
			rawDataSourceRef.value = rs;
			const isArrayRs = isArray(rs);

			let rsList = isArrayRs ? rs : get(rs, listField);
			const rsTotal = isArrayRs ? 0 : get(rs, totalField);

			dataSourceRef.value = rsList;

			// TODO: 修改并设置正确的分页

			emit('fetch-success', {
				list: unref(rsList),
				total: rsTotal,
			});
		} catch (error) {
			emit('fetch-error', error);
			dataSourceRef.value = [];
		} finally {
			setLoading(false);
		}
	}

	function getDataSource() {
		return unref(dataSourceRef);
	}

	function getRawDataSource() {
		return rawDataSourceRef.value;
	}

	async function reload(opts?: FetchParams) {
		return await fetch(opts);
	}

	onMounted(() => {
		unref(propsRef).immediate && fetch();
	});

	return {
		getDataSourceRef,
		getDataSource,
		getRawDataSource,
		getRowKeyRef,
		fetch,
		reload,
	};
}
